package de.fish.tabacha.test;

/** .
   <br> $Id: Arch.java,v 1.1 2002/07/22 14:25:11 sva00-michael Exp $
   @author michael@tabacha.de
   @author $Author: sva00-michael $
   @version $Revision: 1.1 $
*/
public class Arch
{

    /** Constant for arch type: Anything else */
    public static int NORMAL = 0; 
    /** Constant for arch type: Ships */
    public static int SHIP = 1;
    /** Constant for arch type: Bridges */
    public static int BRIDGE = 2;
    /** Constant for arch type: Humans, humanoids, monsters, pcs, npcs */
    public static int LIVING = 3;
    /** Constant for arch type: Passable by ships */
    public static int WATER = 4;
    /** Constant for arch type: Images for the "Kartenspezialdarstellung" */
    public static int SPECIAL = 5;

    private static final String[] TYPES = new String[]{"normal", "ship", "bridge", "living", "water", "special"};


    private int number;
    private int type;
    private boolean transparent;
    private boolean passable;
    

    /** Constructor.
	Attributes are determined by the object number.
	@param number old style object number
    */
    public Arch(int number)
    {
	this.number = number;

	if (number < 1500)
	    { type=NORMAL; passable=true; transparent=true; }
	else if (number < 1950)
	    { type=SHIP; passable=true; transparent=true; }
	else if (number < 2000)
	    { type=BRIDGE; passable=true; transparent=true; }
	else if (number < 3000)
	    { type=NORMAL; passable=false; transparent=true; }
	else if (number < 3500)
	    { type=LIVING; passable=false; transparent=true; }
	else if (number < 4000)
	    { type=WATER; passable=false; transparent=true; }
	else if (number < 5500)
	    { type=NORMAL; passable=true; transparent=false; }
	else if (number < 8000)
	    { type=NORMAL; passable=false; transparent=false; }
	else 
	    { type=SPECIAL; passable=true; transparent=true; }
    }


    /** Returns the number of the object.
     */
    public int getNumber()
    { return number; }

    /** Returns the type of the object.
	See the constants.
    */
    public int getType()
    { return type; }

    /** Returns whether the object is passable ("begehbar").
     */
    public boolean isPassable()
    { return passable; }

    /** Returns whether the object is transparent ("durchsichtig").
     */
    public boolean isTransparent()
    { return transparent; }
    
    /** Returns the arch as XML node.
     */
    public String asXML()
    {
	StringBuffer xml = new StringBuffer();
	xml.append("<arch number=\"")
	    .append(getNumber())
	    .append("\" type=\"")
	    .append(TYPES[getType()])
	    .append("\" passable=\"")
	    .append(isPassable())
	    .append("\" transparent=\"")
	    .append(isTransparent())
	    .append("\"/>")
	    .append(System.getProperty("line.separator"));
	return xml.toString();
    }

    /** Returns the object as crossfire arch file string.
     */
    public String asArc()
    {
	String nl = System.getProperty("line.separator");
	StringBuffer arch = new StringBuffer();
	arch.append("Object ").append(getNumber()).append(nl)
	    .append("name ").append(TYPES[getType()]).append(nl) // name must be set manually
	    .append("face ").append(getNumber()).append(".111").append(nl)
	    .append("no_pick 1").append(nl)
	    .append("is_floor 1").append(nl)
	    .append("editable 8").append(nl); // background
	// type not set
	if (!isPassable())
	    arch.append("no_pass 1").append(nl);
	if (!isTransparent())
	    arch.append("blocksview 1").append(nl);
	arch.append("end").append(nl);
	return arch.toString();
    }

}
