package de.fish.tabacha.test;

import java.io.FileInputStream;
import java.io.File;
import java.io.IOException;

/** Holds information about one map.
    <br /> $Id: Map.java,v 1.1 2002/07/22 14:25:11 sva00-michael Exp $
    @author nils <nils@teha-systems.de>
    @author michael@tabacha.de
    @author $Author: sva00-michael $
    @version $Revision: 1.1 $
*/
public class Map
{
    // Constants -----------------------------------------------------

    // Attributes ----------------------------------------------------

    private String name;
    private int[][] map;
    private int up, down, left, right;


    // Constructors --------------------------------------------------

    /** Constructor.
     */
    public Map(String filename)
	throws IOException
    { this(new File(filename)); }

    /** Constructor.
     */
    public Map(File mapFile)
	throws IOException
    {
	
	this.name = FilenameHelper.cutSuffix(mapFile.getName());
	this.map = new int[30][30];
	
	FileInputStream fileInputStream = new FileInputStream(mapFile);
	
	// read adjacent maps
	up = fileInputStream.read() + 256 * fileInputStream.read();
	down = fileInputStream.read() + 256 * fileInputStream.read();
	left = fileInputStream.read() + 256 * fileInputStream.read();
	right = fileInputStream.read() + 256 * fileInputStream.read();
	
	// read content
	for (int i=0;i<900;i++)
	    map[i / 30][i % 30] = fileInputStream.read()+256*fileInputStream.read();
	
	fileInputStream.close();
    }
    

    // Public --------------------------------------------------------
    
    public String getName()
    { return name; }

    public int getUp()
    { return up; }
    
    public int getDown()
    { return down; }

    public int getLeft()
    { return left; }

    public int getRight()
    { return right; }

    /** Returns a subfield of the current map.
	Elements out of range are set to zero.
	@returns the numbers of the objects on the current map.
    */
    public int[][] getSubfield(int xFrom, int yFrom, int xTo, int yTo)
    {
	if (xFrom > xTo || yFrom > yTo)
	    return null;
	int[][] subfield = new int[xTo-xFrom+1][yTo-yFrom+1];
	for (int i=xFrom; i<=xTo; i++)
	    for (int j=yFrom; j<=yTo; j++)
		subfield[i][j] = ( (i<0 || i>=30 || j<0 || j>=30) ? 0 : map[i][j]);
	return subfield;
    }

    /** Returns the number of the object on the specified square.
     */
    public int getSquare(int x, int y)
    { return map[x][y]; }


    /** Returns the map as XML node.
     */
    public String asXML()
    {
	StringBuffer xml = new StringBuffer();
	xml.append("<map name=\"")
	    .append(getName())
	    .append("\"><links_to up=\"")
	    .append(getUp())
	    .append("\" down=\"")
	    .append(getDown())
	    .append("\" left=\"")
	    .append(getLeft())
	    .append("\" right=\"")
	    .append(getRight())
	    .append("\"/>")
	    .append(System.getProperty("line.separator"))
	    .append("<contains>");
	for (int x=0; x<30; x++)
	    for (int y=0; y<30; y++)
		{
		    xml.append("<square x=\"")
			.append(x)
			.append("\" y=\"")
			.append(y)
			.append("\" arch=\"")
			.append(getSquare(x,y))
			.append("\"/>");
		}
	xml.append("</contains>")
	    .append(System.getProperty("line.separator"))
	    .append("</map>")
	    .append(System.getProperty("line.separator"));
	return xml.toString();
    }

    /** Returns the map as a string in crossfire map file format.
     */
    public String toCrossfire()
    {
	String nl = System.getProperty("line.separator");
	StringBuffer cfmap = new StringBuffer();
	cfmap.append("arch map").append(nl)
	    .append("name ").append(getName()).append(nl)
	    /* 'width','height', was 'x','y': Size of the map. */
	    .append("width 30").append(nl)
	    .append("height 30").append(nl)
	    /* tile_path_<x> - Used with map tiling.  <x> is a number, 1 is north,
	       2 is east, 3 south, 4 west.  This determines what map is tiled
	       in that direction.
	    */
	    .append("tile_path_1 ").append(getUp()).append(nl)
	    .append("tile_path_2 ").append(getLeft()).append(nl)
	    .append("tile_path_3 ").append(getDown()).append(nl)
	    .append("tile_path_4 ").append(getRight()).append(nl)
	    /* 'outdoor' - if set, this is a hint that this is an outdoor map.
	       If this is not set, weather and dawn/dusk will not occur on this
	       map.  It is highly advised that this be set appropriately.
	       no information about this
	    */
	    .append("end").append(nl);
	for (int x=0; x<30; x++)
	    for (int y=0; y<30; y++)
		{
		    cfmap.append("arch ").append(getSquare(x,y)).append(nl);
		    if (x!=0)
			cfmap.append("x ").append(x).append(nl);
		    if (y!=0)
			cfmap.append("y ").append(y).append(nl);
		    cfmap.append("end").append(nl);
		}
	return cfmap.toString();
    }

	
	// Z implementation ----------------------------------------------



	// Y overrides ---------------------------------------------------

	// Package protected ---------------------------------------------

	// Protected -----------------------------------------------------

	// Private -------------------------------------------------------

	// Inner classes -------------------------------------------------

} // Map
