package de.fish.tabacha.test;

import java.io.PrintStream;


/** Eine Abstrakte Klasse von der jeder Test abgeleitet werden soll.
    Der Test muss die Abstrakte Methode runTest() ueberschreiben.
    Ein Beispiel gibt es beide.fish.tabacha.TestGewicht.
    <B>Laut Stylguide MUSS jede Klasse einen Test aufsetzen.</B>
    Dieser sollte den gesammten Quellcode der Klasse mindestens einmal
    ablaufen.<P>
    Dieses Testobjekt ist fuer eine Zeilenbreite von 80 ausgelegt.
    <P> $Id: Test.java,v 1.1 2002/02/26 23:38:58 sva00-michael Exp $
    @see de.fish.tabacha.GewichtTest
    @author ?
    @author $Author: sva00-michael $
    @version $Revision: 1.1 $
*/      
public abstract class Test {

    /** Der Ausgabekanal im Normal System.out Diser sollte von
	abgeleiteten Objekten verwendet werden um Ausgaben zu machen.
	Dies ermoeglicht das spaetere Umleiten in Dateien oder 
	aehnlichem */
    public PrintStream out;

    /** Die Nummer des Tests. Wird von teste gebraucht*/
    private int nr=0;

    /** Die Anzahl der gepassden Tests. */
    private int passed=0;

    /** Die Anzahl der failden Tests. */
    private int failed=0;

    /** Das wird ausgegeben wenn ein TEST okay ist */
    public String okaystr="   OKAY  ";

    /** Das wird ausgegeben wenn ein TEST nicht okay ist */
    public String failstr=" *FAILED*";

    /** Standart Konstruktur setzt den Ausgabestream auf System.out.
	@see java.lang.System */
    public  Test() {
	out=System.out;
    }

    /**  Konstruktur setzt den Ausgabestream auf myout
	 @param myout Stream z.B. System.err oder System.out ... */
    public Test(PrintStream myout) {
	out=myout;
    }

    /**  Diese Routine testet eine Bedingung auf Wahrheit 
	 @param s Beschreibung des Tests im Normalfall das gleiche wie b nur
	 in Anfuehrungsstrichen. 
	 @param b Ein Test. Er muss entweder WAHR oder FALSCH sein.
	 Bei FALSCH ist der Test gescheitert.
    */

    public void teste(String s, boolean b) {
	
	//	System.out.print(_i);
	int i;
	nr++;
	String k="";
	String nrstr= new Integer(nr).toString();
	for (i=nrstr.length(); i<5; i++) {
	    k=k.concat(" ");
	}
	out.print(k);
	out.print(nrstr);
	out.print(". ");
	out.print(s);
	k="";
	for (i=s.length(); i<70; i++) {
	    k=k.concat(" ");
	}
	out.print(k);
	if (b) {
	    out.println(okaystr);
	    passed++;
	} else {
	    out.println(failstr);
	    failed++;
	}
	
    }

    /** Gibt einen Strich aus */
    public  void printRule() {
	out.println("=======================================================================================");
    } 
    
    /** Das Testprogramm fuer die Klasse. 
	Muss von der abgeleiteten Klasse ueberschrieben werden. */
    public abstract void runTest() ;
   
    /** Gibt einen Anfang zum Gesamten-Test aus. 
	wird vonde.fish.tabacha.Teste.main() aufgerufen und
	muss im Normalfall NICHT ueberschrieben werden.
	@see de.fish.tabacha.Teste
    */
    public void printAnfang(String classname) 
    {
	printRule();
	out.print("                               Beginn des Tests von: ");
    	out.println(classname);
	printRule();
    }

    /** Gibt eine Zusammenfassung zum Gesamttest aus. 
	wird vonde.fish.tabacha.Teste.main() aufgerufen und
	muss im Normalfall NICHT ueberschrieben werden.
	@see de.fish.tabacha.Teste
    */
    public void printZusammenfassung() {
	printRule();
	out.println("                                  Zusammenfassung");
	printRule();
	out.print("                                                   # TESTS  = ");
	out.println(nr);
	out.print("                                                   # OK     = ");
	out.println(passed);	
	if (failed > 0) {
	    out.print("                                                   # FEHLER = ");
	    out.println(failed);
	}
	printRule();
    }
    
}






